#!/bin/bash
#
# Scalr setup tool.
#
# * Wed May 22 2013 KAJIKAWA Haruhiko <hkajikawa@idcf.jp> - v2
# - scalr-4.1.0 support
#
# * Mon Jan 21 2013 KAJIKAWA Haruhiko <hkajikawa@idcf.jp> - v1
# - first release.
#

VERSION="2"

###########################
## Common Function
###########################
function error() {
  echo "error occurred. sorry..."
  echo
  exit 1
}

function backup_file() {
  local FILE=$1
  rm -rf $FILE.origin   > /dev/null 2>&1 || return 1
  cp $FILE $FILE.origin > /dev/null 2>&1 || return 1
}

###########################
## MySQL
###########################
function mysql_start_and_secure_setting() {
  echo "starting mysqld..."
  service mysqld start > /dev/null 2>&1 || return 1
  while [ -z "$ROOTPW" ]; do
    read -p "mysql root user password = ? " ROOTPW
  done
  cat <<- EOF | mysql -u root
	UPDATE mysql.user SET Password=PASSWORD('$ROOTPW') WHERE User='root';
	DELETE FROM mysql.user WHERE User='';
	DELETE FROM mysql.user WHERE User='root' AND Host NOT IN ('localhost', '127.0.0.1', '::1');
	DROP DATABASE test;
	DELETE FROM mysql.db WHERE Db='test' OR Db='test\_%';
	FLUSH PRIVILEGES;
	EOF
}

function mysql_set_checkconfig() {
  echo "setup chkconfig mysqld..."
  chkconfig mysqld on > /dev/null 2>&1 || return 1
}

###########################
## Scalr Setup
###########################
function scalr_set_php_ini() {
  echo "setup sample php.ini..."
  backup_file /etc/php.ini
  sed -i -e "s/^expose_php.*=.*/expose_php = Off/" /etc/php.ini
  sed -i -e "s/^short_open_tag.*=.*/short_open_tag = On/" /etc/php.ini
  if [ -e /etc/sysconfig/clock ]; then
    . /etc/sysconfig/clock
  fi
  if [ -n "$ZONE" ]; then
    PHP_ZONE=$(echo ${ZONE/\//\\/})
  fi
  if [ -n "$PHP_ZONE" ]; then
    while [ -z "$PHP_ZONE" ]; do
      read -p "php.ini date.timezone = ? " PHP_ZONE
    done
  fi
  sed -i -e "s/^;date.timezone.*=.*/date.timezone = $PHP_ZONE/" /etc/php.ini
}

function scalr_set_autogen_domain() {
  echo "setup automatically generate domain..."
  backup_file /usr/share/scalr/src/structs/struct.CONFIG.php
  while [ -z "$SCALR_AUTOGEN_DOMAIN" ]; do
    read -p "auto generate domain = ? " SCALR_AUTOGEN_DOMAIN
  done
  SCALR_AUTOGEN_DOMAIN=$( echo $SCALR_AUTOGEN_DOMAIN | sed -e "s/\.*$//" )

  sed -i -e "s/\(^.*public static \$DNS_TEST_DOMAIN_NAME = \)'example.ws';/\1'$SCALR_AUTOGEN_DOMAIN';/" /usr/share/scalr/src/structs/struct.CONFIG.php

  echo "setup sample named.conf & zone file..."
  while [ -z "$SCALR_SOA_MNAME" ]; do
    read -p "NS(SOA MNAME) FQDN for auto generate domain = ? " SCALR_SOA_MNAME
  done
  SCALR_SOA_MNAME=$( echo $SCALR_SOA_MNAME | sed -e "s/\.*$//" )
  while [ -z "$SCALR_SOA_RNAME" ]; do
    read -p "EMAIL(SOA RNAME) for auto generate domain = ? " SCALR_SOA_RNAME
  done
  SCALR_SOA_RNAME=$( echo $SCALR_SOA_RNAME | tr "@" "." | sed -e "s/\.*$//" )

  sed -i -e "s/\(^.*public static \$DEF_SOA_PARENT\);/\1 = '$SCALR_SOA_MNAME';/" /usr/share/scalr/src/structs/struct.CONFIG.php

  echo "setup sample named.conf & zone file..."
  backup_file /etc/named.conf
  sed -i -e "s/\tlisten-on/#\tlisten-on/" /etc/named.conf
  echo "include \"/var/named/$SCALR_AUTOGEN_DOMAIN.conf\";" >> /etc/named.conf
  echo 'include "/var/named/client_zones/zones.include";' >> /etc/named.conf

  cat <<-EOF > /var/named/$SCALR_AUTOGEN_DOMAIN.conf
	zone "$SCALR_AUTOGEN_DOMAIN" IN {
	    type master;
	    file "$SCALR_AUTOGEN_DOMAIN.zone";
	    allow-update { none; };
	    allow-query { any; };
	    #allow-transfer { zzz.zzz.zzz.zzz; };
	    notify yes;
	};
	EOF
  cat <<-EOF > /var/named/$SCALR_AUTOGEN_DOMAIN.zone
	@   14400 IN SOA $SCALR_SOA_MNAME. $SCALR_SOA_RNAME. (
	    0
	    14400
	    7200
	    86400
	    300 )
	    NS  $SCALR_SOA_MNAME.
	EOF
  chown root:named /var/named/$SCALR_AUTOGEN_DOMAIN.*
  chmod 640 /var/named/$SCALR_AUTOGEN_DOMAIN.*
}

function scalr_set_named_checkconfig() {
  echo "setup chkconfig named..."
  chkconfig named on  > /dev/null 2>&1 || return 1
  echo "starting named..."
  service named start > /dev/null 2>&1 || return 1
}

function scalr_set_mysql_db() {
  echo "setup mysql scalr user & scalr db & scalr table & data..."
  while [ -z "$MYSQL_SCALR_PASSWD" ]; do
    read -p "mysql scalr user password = ? " MYSQL_SCALR_PASSWD
  done
  mysql -u root -p$ROOTPW -e "create database scalr; grant all on scalr.* to scalr@localhost identified by '$MYSQL_SCALR_PASSWD';flush privileges;" || return 1
  mysql -u scalr -p$MYSQL_SCALR_PASSWD scalr < /usr/share/doc/scalr-*/structure.sql || return 1
  mysql -u scalr -p$MYSQL_SCALR_PASSWD scalr < /usr/share/doc/scalr-*/data.sql      || return 1
}

function scalr_set_config_ini() {
  echo "setup scalr config.ini..."
  backup_file /usr/share/scalr/etc/config.ini

  while [ -z "$PUBLIC_IP" ]; do
    read -p "public ip or fqdn for sclar = ? " PUBLIC_IP
  done

  cat <<-EOF > /usr/share/scalr/etc/config.ini
	[db]
	driver=mysqli
	host = "localhost"
	name = "scalr"
	user = "scalr"
	pass = "$MYSQL_SCALR_PASSWD"
	
	[statistics]
	rrd_default_font_path   = "/usr/share/fonts/dejavu/DejaVuSansMono.ttf"
	rrd_db_dir              = "/var/www/scalr/rrddata"
	rrd_stats_url           = "http://$PUBLIC_IP/graphics/%fid%/%rn%_%wn%."
	rrd_graph_storage_path  = "/var/www/scalr/graphics"
	EOF

  mysql -u scalr -p$MYSQL_SCALR_PASSWD scalr -e "UPDATE config SET value = '$PUBLIC_IP' WHERE \`key\`='eventhandler_url'" || return 1
  mysql -u scalr -p$MYSQL_SCALR_PASSWD scalr -e "UPDATE config SET value = 'http' WHERE \`key\`='http_proto'" || return 1
}

function scalr_set_scalr_admin_pw() {
  echo "setup scalr admin password..."
  while [ -z "$SCALR_ADMIN_PASSWD" ]; do
    read -p "scalr admin user password = ? " SCALR_ADMIN_PASSWD
  done
  SCALR_ADMIN_PASSWD_SHA256=$( echo -n "$SCALR_ADMIN_PASSWD" | sha256sum | cut -d' ' -f 1 )
  mysql -u scalr -p$MYSQL_SCALR_PASSWD scalr -e "update account_users set password='$SCALR_ADMIN_PASSWD_SHA256' where id=1" || return 1
}

function scalr_set_cron() {
  echo "setup root & scalr user cron..."
  cat /usr/share/doc/scalr-*/cron-scalr-scalr >> /var/spool/cron/scalr
  cat /usr/share/doc/scalr-*/cron-scalr-root  >> /var/spool/cron/root
  service crond restart > /dev/null 2>&1 || return 1
}

function scalr_set_checkconfig() {
  echo "starting httpd..."
  echo "setup chkconfig httpd..."
  chkconfig httpd  on  > /dev/null 2>&1 || return 1
  service httpd start  > /dev/null 2>&1 || return 1
}

###########################
## Main
###########################
while [ -z "$SELECT" ]; do
  clear
  echo '### "SCALR" setup tool ###'
  echo ""
  echo 'You can setting the "SCALR" in this setup tool.'
  echo ""

  echo "Select and Setup."
  select SELECT_NO in 'Scalr setup' 'exit'; do
    case $REPLY in
      1 )
        mysql_start_and_secure_setting || error
        mysql_set_checkconfig       || error
        scalr_set_mysql_db          || error
        scalr_set_config_ini        || error
        scalr_set_scalr_admin_pw    || error
        scalr_set_autogen_domain    || error
        scalr_set_named_checkconfig || error
        scalr_set_cron              || error
        scalr_set_php_ini           || error
        scalr_set_checkconfig       || error
        echo
        echo "Setup tool installed the sample configuration."
        echo "Please complete setting to access the WebGUI."
	echo "http://$PUBLIC_IP/"
        echo "  Username: admin"
        echo "  Password: $SCALR_ADMIN_PASSWD"
        echo "Please create a Saclr user."
        echo "To install the MindTerm, please refer to the \"/usr/share/doc/scalr-*/README.idcf.scalr\"."
        echo
        break 2
        ;;
      2 )
        echo "$REPLY ) $SELECT_NO setup canceled."
        exit 2
        ;;
      * )
        break
    esac
  done
done
echo
echo "$REPLY ) $SELECT_NO setup finished!"
echo

